#ifndef __EA_H
#define __EA_H

#include<stdint.h>
#include<stddef.h>
#include<sys/syscall.h>
#include<sys/types.h>
#include<sys/unistd.h>

/*take this out when compiler support is common*/
#if (!defined( __EA32__ ) && !defined( __EA64__ ))
#warning  "need __ea support in compiler to compile to take advantage of libea features"
#define __ea
#define __EA32__
#endif

#ifdef __EA64__

typedef uint64_t size_ea_t;
typedef uint64_t key_ea_t;

#define MAP_FAILED_EA ((__ea void *) -1LL)

/* Memory Mappings */
__ea void *__mmap_ea64(__ea void *start, size_ea_t length, int prot, int flags, int fd, off_t offset);
static inline __ea void *mmap_ea(__ea void *start, size_ea_t length, int prot, int flags, int fd, off_t offset){
  return(__mmap_ea64(start,length,prot,flags,fd,offset));
}

int __munmap_ea64(__ea void *start, size_ea_t length);
static inline int munmap_ea(__ea void *start, size_ea_t length){
  return(__munmap_ea64(start,length));
}

__ea void* __mremap_ea64(__ea void *old_address, size_ea_t old_size, size_ea_t new_size, unsigned long flags);
static inline __ea void* mremap_ea(__ea void *old_address, size_ea_t old_size, size_ea_t new_size, unsigned long flags){
  return(__mremap_ea64(old_address, old_size, new_size, flags));
}

int __msync_ea64(__ea void *start, size_ea_t length, int flags);
static inline int msync_ea(__ea void *start, size_ea_t length, int flags){
  return(__msync_ea64(start, length, flags));
}

/* EA memory mangement functions */
__ea void* __calloc_ea64(size_ea_t nmemb, size_ea_t length);
static inline __ea void* calloc_ea(size_ea_t nmemb, size_ea_t length){
  return __calloc_ea64(nmemb, length);
}

void __free_ea64(__ea void *ptr);
static inline void free_ea(__ea void *ptr){
  return __free_ea64(ptr);
}

__ea void* __malloc_ea64(size_ea_t size);
static inline __ea void* malloc_ea(size_ea_t size){
  return __malloc_ea64(size);
}

__ea void* __realloc_ea64(__ea void *ptr, size_ea_t size);
static inline __ea void* realloc_ea(__ea void *ptr, size_ea_t size){
  return __realloc_ea64(ptr, size);
}

/*String Copying Functions*/

__ea void *__memcpy_ea64(__ea void *dest, __ea const void *src, size_ea_t n);
static inline __ea void *memcpy_ea(__ea void *dest, __ea const void *src, size_ea_t n){
  return __memcpy_ea64(dest, src, n);
}

__ea void *__memmove_ea64(__ea void *dest, __ea const void *src, size_ea_t n);
static inline __ea void *memmove_ea(__ea void *dest, __ea const void *src, size_ea_t n){
  return __memmove_ea64(dest, src, n);
}

__ea char *__strcpy_ea64(__ea char *dest, __ea const char *src);
static inline __ea char *strcpy_ea(__ea char *dest, __ea const char *src){
  return __strcpy_ea64(dest, src);
}

__ea char *__strncpy_ea64(__ea char *dest, __ea const char *src, size_ea_t n);
static inline __ea char *strncpy_ea(__ea char *dest, __ea const char *src, size_ea_t n){
  return __strncpy_ea64(dest, src, n);
}

/* Concatenation functions */
__ea char *__strcat_ea64(__ea char *dest, __ea const char *src);
static inline __ea char *strcat_ea(__ea char *dest, __ea const char *src){
  return __strcat_ea64(dest, src);
}

__ea char *__strncat_ea64(__ea char *dest, __ea const char *src, size_ea_t n);
static inline __ea char *strncat_ea(__ea char *dest, __ea const char *src, size_ea_t n){
  return __strncat_ea64(dest, src, n);
}

/*Comparison functions*/
int __memcmp_ea64(__ea void *s1, __ea const void *s2, size_ea_t n);
static inline int memcmp_ea(__ea void *s1, __ea const void *s2, size_ea_t n){
  return __memcmp_ea64(s1, s2 ,n);
}

int __strcmp_ea64(__ea char *s1, __ea const char *s2);
static inline int strcmp_ea(__ea char *s1, __ea const char *s2){
  return __strcmp_ea64(s1, s2);
}

int __strncmp_ea64(__ea void *s1, __ea const void *s2, size_ea_t n3);
static inline int strncmp_ea(__ea void *s1, __ea const void *s2, size_ea_t n3){
  return __strncmp_ea64(s1, s2, n3);
}


/*Search functions*/
__ea void *__memchr_ea64(__ea const void *s, int c, size_ea_t n);
static inline __ea void *memchr_ea(__ea const void *s, int c, size_ea_t n){
  return __memchr_ea64(s, c, n);
}
__ea char *__strchr_ea64(__ea const char *s, int c);
static inline __ea char *strchr_ea(__ea const char *s, int c){
  return __strchr_ea64(s,c);
}

size_ea_t __strcspn_ea64(__ea const char *s, const char *reject);
static inline size_ea_t strcspn_ea(__ea const char *s, const char *reject){
  return __strcspn_ea64(s, reject);
}

__ea char *__strpbrk_ea64(__ea const char *s, const char *accept);
static inline __ea char *strpbrk_ea(__ea const char *s, const char *accept){
  return __strpbrk_ea64(s, accept);
}

__ea char *__strrchr_ea64(__ea const char *s, int c);
static inline __ea char *strrchr_ea(__ea const char *s, int c){
  return __strrchr_ea64(s, c);
}

size_ea_t __strspn_ea64(__ea const char *s, const char* accept);
static inline size_ea_t strspn_ea(__ea const char *s, const char* accept){
  return __strspn_ea64(s, accept);
}

__ea char * __strstr_ea64(__ea const char *s1, __ea const char *s2);
static inline __ea char * strstr_ea(__ea const char *s1, __ea const char *s2){
  return __strstr_ea64(s1, s2);
}

/* Misc functions */
__ea void *__memset_ea64(__ea void *dest, int c, size_ea_t n);
static inline __ea void *memset_ea(__ea void *dest, int c, size_ea_t n){
  return __memset_ea64(dest, c, n);
}

size_ea_t __strlen_ea64(__ea const char *s);
static inline size_ea_t strlen_ea(__ea const char *s){
  return __strlen_ea64(s);
}

#else

typedef uint32_t size_ea_t;
typedef uint32_t key_ea_t;

#define MAP_FAILED_EA ((__ea void *) -1)

/* Memory Mappings */
__ea void *__mmap_ea32(__ea void *start, size_ea_t length, int prot, int flags, int fd, off_t offset);
static inline __ea void *mmap_ea(__ea void *start, size_ea_t length, int prot, int flags, int fd, off_t offset){
  return __mmap_ea32(start,length,prot,flags,fd,offset);
}

int __munmap_ea32(__ea void *start, size_ea_t length);
static inline int munmap_ea(__ea void *start, size_ea_t length){
  return __munmap_ea32(start,length);
}

__ea void* __mremap_ea32(__ea void *old_address, size_ea_t old_size, size_ea_t new_size, unsigned long flags);
static inline __ea void* mremap_ea(__ea void *old_address, size_ea_t old_size, size_ea_t new_size, unsigned long flags){
  return __mremap_ea32(old_address, old_size, new_size, flags);
}

int __msync_ea32(__ea void *start, size_ea_t length, int flags);
static inline int msync_ea(__ea void *start, size_ea_t length, int flags){
  return __msync_ea32(start, length, flags);
}
/* EA memory mangement functions */
__ea void* __calloc_ea32(size_ea_t nmemb, size_ea_t length);
static inline __ea void* calloc_ea(size_ea_t nmemb, size_ea_t length){
  return __calloc_ea32(nmemb, length);
}

void __free_ea32(__ea void *ptr);
static inline void free_ea(__ea void *ptr){
  return __free_ea32(ptr);
}

__ea void* __malloc_ea32(size_ea_t size);
static inline __ea void* malloc_ea(size_ea_t size){
  return __malloc_ea32(size);
}

__ea void* __realloc_ea32(__ea void *ptr, size_ea_t size);
static inline __ea void* realloc_ea(__ea void *ptr, size_ea_t size){
  return __realloc_ea32(ptr, size);
}


/*Copying functions*/
__ea void *__memcpy_ea32(__ea void *dest, __ea const void *src, size_ea_t n);
static inline __ea void *memcpy_ea(__ea void *dest, __ea const void *src, size_ea_t n){
  return __memcpy_ea32(dest, src, n);
}

__ea void *__memmove_ea32(__ea void *dest, __ea const void *src, size_ea_t n);
static inline __ea void *memmove_ea(__ea void *dest, __ea const void *src, size_ea_t n){
  return __memmove_ea32(dest, src, n);
}

__ea char *__strcpy_ea32(__ea char *dest, __ea const char *src);
static inline __ea char *strcpy_ea(__ea char *dest, __ea const char *src){
  return __strcpy_ea32(dest, src);
}

__ea char *__strncpy_ea32(__ea char *dest, __ea const char *src, size_ea_t n);
static inline __ea char *strncpy_ea(__ea char *dest, __ea const char *src, size_ea_t n){
  return __strncpy_ea32(dest, src, n);
}


/* Concatenation functions */
__ea char *__strcat_ea32(__ea char *dest, __ea const char *src);
static inline __ea char *strcat_ea(__ea char *dest, __ea const char *src){
  return __strcat_ea32(dest, src);
}

__ea char *__strncat_ea32(__ea char *dest, __ea const char *src, size_ea_t n);
static inline __ea char *strncat_ea(__ea char *dest, __ea const char *src, size_ea_t n){
  return __strncat_ea32(dest, src, n);
}

/*Comparison functions*/
int __memcmp_ea32(__ea void *s1, __ea const void *s2, size_ea_t n);
static inline int memcmp_ea(__ea void *s1, __ea const void *s2, size_ea_t n){
  return __memcmp_ea32(s1, s2 ,n);
}

int __strcmp_ea32(__ea char *s1, __ea const char *s2);
static inline int strcmp_ea(__ea char *s1, __ea const char *s2){
  return __strcmp_ea32(s1, s2);
}

int __strncmp_ea32(__ea void *s1, __ea const void *s2, size_ea_t n3);
static inline int strncmp_ea(__ea void *s1, __ea const void *s2, size_ea_t n3){
  return __strncmp_ea32(s1, s2, n3);
}


/*Search functions*/
__ea void *__memchr_ea32(__ea const void *s, int c, size_ea_t n);
static inline __ea void *memchr_ea(__ea const void *s, int c, size_ea_t n){
  return __memchr_ea32(s, c, n);
}

__ea char *__strchr_ea32(__ea const char *s, int c);
static inline __ea char *strchr_ea(__ea const char *s, int c){
  return __strchr_ea32(s, c);
}

size_ea_t __strcspn_ea32(__ea const char *s, const char *reject);
static inline size_ea_t strcspn_ea(__ea const char *s, const char *reject){
  return __strcspn_ea32(s, reject);
}

__ea char *__strpbrk_ea32(__ea const char *s, const char *accept);
static inline __ea char *strpbrk_ea(__ea const char *s, const char *accept){
  return __strpbrk_ea32(s, accept);
}

__ea char *__strrchr_ea32(__ea const char *s, int c);
static inline __ea char *strrchr_ea(__ea const char *s, int c){
  return __strrchr_ea32(s, c);
}

size_ea_t __strspn_ea32(__ea const char *s, const char* accept);
static inline size_ea_t strspn_ea(__ea const char *s, const char* accept){
  return __strspn_ea32(s, accept);
}

__ea char * __strstr_ea32(__ea const char *s1, __ea const char *s2);
static inline __ea char * strstr_ea(__ea const char *s1, __ea const char *s2){
  return __strstr_ea32(s1, s2);
}

/* Misc functions */
__ea void *__memset_ea32(__ea void *dest, int c, size_ea_t n);
static inline __ea void *memset_ea(__ea void *dest, int c, size_ea_t n){
  return __memset_ea32(dest, c, n);
}

size_ea_t __strlen_ea32(__ea const char *s);
static inline size_ea_t strlen_ea(__ea const char *s){
  return __strlen_ea32(s);
}

#endif


#if defined( __EA64__ ) && defined( __EA32__ )
#error can not do 32 and 64 bit at once
#endif

#endif /* __EA_H*/
